@@ -1,4 +1,5 @@
-/* Copyright 2002-2004 The Apache Software Foundation
+/* Copyright 2002-2005 The Apache Software Foundation or its licensors, as
+ * applicable.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -58,7 +59,7 @@ static int workers_may_exit = 0;
 static unsigned int g_blocked_threads = 0;
 static HANDLE max_requests_per_child_event;
 
-
+static apr_thread_mutex_t  *child_lock;
 static apr_thread_mutex_t  *qlock;
 static PCOMP_CONTEXT qhead = NULL;
 static PCOMP_CONTEXT qtail = NULL;
@@ -145,6 +146,7 @@ AP_DECLARE(PCOMP_CONTEXT) mpm_get_completion_context(void)
                  */
                 apr_allocator_t *allocator;
 
+                apr_thread_mutex_lock(child_lock);
                 context = (PCOMP_CONTEXT) apr_pcalloc(pchild, sizeof(COMP_CONTEXT));
   
                 context->Overlapped.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
@@ -152,6 +154,8 @@ AP_DECLARE(PCOMP_CONTEXT) mpm_get_completion_context(void)
                     /* Hopefully this is a temporary condition ... */
                     ap_log_error(APLOG_MARK,APLOG_WARNING, apr_get_os_error(), ap_server_conf,
                                  "mpm_get_completion_context: CreateEvent failed.");
+
+                    apr_thread_mutex_unlock(child_lock);
                     return NULL;
                 }
  
@@ -163,6 +167,8 @@ AP_DECLARE(PCOMP_CONTEXT) mpm_get_completion_context(void)
                     ap_log_error(APLOG_MARK,APLOG_WARNING, rv, ap_server_conf,
                                  "mpm_get_completion_context: Failed to create the transaction pool.");
                     CloseHandle(context->Overlapped.hEvent);
+
+                    apr_thread_mutex_unlock(child_lock);
                     return NULL;
                 }
                 apr_allocator_owner_set(allocator, context->ptrans);
@@ -170,6 +176,8 @@ AP_DECLARE(PCOMP_CONTEXT) mpm_get_completion_context(void)
                 context->accept_socket = INVALID_SOCKET;
                 context->ba = apr_bucket_alloc_create(pchild);
                 apr_atomic_inc(&num_completion_contexts); 
+
+                apr_thread_mutex_unlock(child_lock);
                 break;
             }
         } else {
@@ -408,6 +416,7 @@ static PCOMP_CONTEXT win9x_get_connection(PCOMP_CONTEXT context)
     if (context == NULL) {
         /* allocate the completion context and the transaction pool */
         apr_allocator_t *allocator;
+        apr_thread_mutex_lock(child_lock);
         context = apr_pcalloc(pchild, sizeof(COMP_CONTEXT));
         apr_allocator_create(&allocator);
         apr_allocator_max_free_set(allocator, ap_max_mem_free);
@@ -415,6 +424,7 @@ static PCOMP_CONTEXT win9x_get_connection(PCOMP_CONTEXT context)
         apr_allocator_owner_set(allocator, context->ptrans);
         apr_pool_tag(context->ptrans, "transaction");
         context->ba = apr_bucket_alloc_create(pchild);
+        apr_thread_mutex_unlock(child_lock);
     }
     
     while (1) {
@@ -682,7 +692,7 @@ static void worker_main(long thread_num)
     ap_sb_handle_t *sbh;
 
     ap_log_error(APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, ap_server_conf,
-                 "Child %d: Worker thread %d starting.", my_pid, thread_num);
+                 "Child %d: Worker thread %ld starting.", my_pid, thread_num);
     while (1) {
         conn_rec *c;
         apr_int32_t disconnected;
@@ -743,7 +753,7 @@ static void worker_main(long thread_num)
                                         (request_rec *) NULL);
 
     ap_log_error(APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, ap_server_conf,
-                 "Child %d: Worker thread %d exiting.", my_pid, thread_num);
+                 "Child %d: Worker thread %ld exiting.", my_pid, thread_num);
 }
 
 
@@ -875,6 +885,8 @@ void child_main(apr_pool_t *pconf)
     ap_log_error(APLOG_MARK,APLOG_NOTICE, APR_SUCCESS, ap_server_conf, 
                  "Child %d: Starting %d worker threads.", my_pid, ap_threads_per_child);
     child_handles = (HANDLE) apr_pcalloc(pchild, ap_threads_per_child * sizeof(int));
+    apr_thread_mutex_create(&child_lock, APR_THREAD_MUTEX_DEFAULT, pchild);
+
     while (1) {
         for (i = 0; i < ap_threads_per_child; i++) {
             int *score_idx;
@@ -897,9 +909,11 @@ void child_main(apr_pool_t *pconf)
             /* Save the score board index in ht keyed to the thread handle. We need this 
              * when cleaning up threads down below...
              */
+            apr_thread_mutex_lock(child_lock);
             score_idx = apr_pcalloc(pchild, sizeof(int));
             *score_idx = i;
             apr_hash_set(ht, &child_handles[i], sizeof(HANDLE), score_idx);
+            apr_thread_mutex_unlock(child_lock);
         }
         /* Start the listener only when workers are available */
         if (!listener_started && threads_created) {
@@ -1067,7 +1081,8 @@ void child_main(apr_pool_t *pconf)
     /* Kill remaining threads off the hard way */
     if (threads_created) {
         ap_log_error(APLOG_MARK,APLOG_NOTICE, APR_SUCCESS, ap_server_conf, 
-                     "Child %d: Terminating %d threads that failed to exit.", my_pid);
+                     "Child %d: Terminating %d threads that failed to exit.", 
+                     my_pid, threads_created);
     }
     for (i = 0; i < threads_created; i++) {
         int *score_idx;
@@ -1082,6 +1097,8 @@ void child_main(apr_pool_t *pconf)
 
     CloseHandle(allowed_globals.jobsemaphore);
     apr_thread_mutex_destroy(allowed_globals.jobmutex);
+    apr_thread_mutex_destroy(child_lock);
+
     if (use_acceptex) {
         apr_thread_mutex_destroy(qlock);
         CloseHandle(qwait_event);
